<?php
/*
Description: Adds a WYSIWYG widget based on the standard TinyMCE WordPress visual editor.
Author: Black Studio
Author URI: http://www.blackstudio.it
License: GPL2

Modified by Hillipino http://hillipino.com

*/

	global $multi_widget_version;
	global $nmp_multi_widget_widget_dev_mode;
	
	$multi_widget_version = '1.1';
	$nmp_multi_widget_widget_dev_mode = false;

// The Widget

	class nmp_multi_widget extends WP_Widget {

		function __construct() {
		
			$widget_ops 	= array( 'classname' => 'nmp_multi', 'description' => __( 'A visual text or HTML editor.', THEMENAME ) );
			$control_ops 	= array( 'width' => 800, 'height' => 800 );
			parent::__construct( 'nmp_multi_widget', __( 'Text Widget', THEMENAME ), $widget_ops, $control_ops);
			
		}

		function widget( $args, $instance ) {
		
			if ( get_option('embed_autourls') ) {
			
				$wp_embed = $GLOBALS['wp_embed'];
				add_filter( 'widget_text', array( $wp_embed, 'run_shortcode' ), 8 );
				add_filter( 'widget_text', array( $wp_embed, 'autoembed' ), 8 );
				
			}		
		
			extract($args);
			
			$text 			= apply_filters( 'widget_text', $instance['text'], $instance );
			
			$multiTitle		= '';
			$multiContainer = '';
			
			if ( isset( $instance['title']) ) { $multiTitle = apply_filters('title',$instance['title']); } 
			if ( isset( $instance['sub_title']) ) { $subtitle = esc_attr($instance['sub_title']); }  else { $subtitle = ''; }
			if ( isset( $instance['nmp_multi_container']) ) { $multiContainer = apply_filters('nmp_multi_container',$instance['nmp_multi_container']); }
			
			
			if( function_exists( 'icl_t' )) {
			
				$title 	= icl_t( "Widgets", 'widget title - ' . md5 ( $title ), $title, $hasTranslation, true );
				$text 	= icl_t( "Widgets", 'widget body - ' . $this->id_base . '-' . $this->number /*md5 ( $text )*/, $text, $hasTranslation, true );
				
			}
			
			$text = do_shortcode( $text );
			
			echo $before_widget;
			
			echo '<div class="obj-text">';			
				
			if ( !empty( $multiTitle ) ) { echo $before_title . $multiTitle . $after_title; } 
			
			if ( $subtitle )
				echo '<p class="subtitle">' . $subtitle . '</p>';
			else 
				echo '<br style="margin-bottom: 1em" />';			
				
			echo $text;

			echo '</div>';
			
			echo $after_widget;
			
		}

		function update( $new_instance, $old_instance ) {
		
			$instance 							= $old_instance;
			$instance['title'] 					= strip_tags($new_instance['title']);
			$instance['sub_title'] 					= strip_tags($new_instance['sub_title']);
			
			if ( current_user_can('unfiltered_html') )
				$instance['text'] =  $new_instance['text'];
			else
				$instance['text'] = stripslashes( wp_filter_post_kses( addslashes($new_instance['text']) ) ); // wp_filter_post_kses() expects slashed
				
			$instance['type'] = strip_tags($new_instance['type']);
			
			if( function_exists( 'icl_register_string' )) {
				
				icl_register_string( "Widgets", 'widget body - ' . $this->id_base . '-' . $this->number  /* md5 ( apply_filters( 'widget_text', $instance['text'] ))*/, apply_filters( 'widget_text', $instance['text'] ));
			}
			
			return $instance;
			
		}

		function form( $instance ) {
		
			$instance 						= wp_parse_args( (array) $instance, array( 'title' => '', 'sub_title' => '', 'text' => '', 'type' => 'visual' ) );
			$title 							= strip_tags($instance['title']);
			$subtitle 						= strip_tags($instance['sub_title']);
			
			if (function_exists('esc_textarea')) {
			
				$text = esc_textarea($instance['text']);
				
			} else {
			
				$text = stripslashes( wp_filter_post_kses( addslashes( $instance['text'] ) ) );
				
			}
			
			$type = esc_attr($instance['type']);
			
			if (get_bloginfo('version') < "3.5") {
			
				$toggle_buttons_extra_class = "editor_toggle_buttons_legacy";
				$media_buttons_extra_class = "editor_media_buttons_legacy";
				
			} else {
			
				$toggle_buttons_extra_class = "wp-toggle-buttons";
				$media_buttons_extra_class = "wp-media-buttons";
				
			}			
			
			?>
			
			<input id="<?php echo $this->get_field_id('type'); ?>" name="<?php echo $this->get_field_name('type'); ?>" type="hidden" value="<?php echo esc_attr($type); ?>" />
			
			<p>
				<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', THEMENAME); ?></label>
				<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
			</p>
			
			<p>
				<label for="<?php echo $this->get_field_id('sub_title'); ?>"><?php _e( 'Widget Sub Title' , THEMENAME ); ?></label>
				<input id="<?php echo $this->get_field_id('sub_title'); ?>" name="<?php echo $this->get_field_name('sub_title'); ?>" type="text" class="widefat" value="<?php echo $subtitle;?>" />
			</p> 			

			<div class="editor_toggle_buttons hide-if-no-js <?php echo $toggle_buttons_extra_class; ?>">
				<a id="widget-<?php echo $this->id_base; ?>-<?php echo $this->number; ?>-html"<?php if ($type == 'html') {?> class="active"<?php }?>><?php _e('HTML', THEMENAME); ?></a>
				<a id="widget-<?php echo $this->id_base; ?>-<?php echo $this->number; ?>-visual"<?php if($type == 'visual') {?> class="active"<?php }?>><?php _e('Visual', THEMENAME); ?></a>
			</div>
			
			<div class="editor_media_buttons hide-if-no-js <?php echo $media_buttons_extra_class; ?>">
				<?php	do_action( 'media_buttons' ); ?>
			</div>
			
			<div class="editor_container">
				<textarea class="widefat" rows="20" cols="40" id="<?php echo $this->get_field_id('text'); ?>" name="<?php echo $this->get_field_name('text'); ?>"><?php echo $text; ?></textarea>
			</div>
			
			<?php
	
			
		}
		
	}

// Initialize Widget

	function nmp_multi_widget_init() {
	
		if ( !is_blog_installed() )
			return;
			
		register_widget('nmp_multi_widget');
		
	}
	
	add_action('widgets_init', 'nmp_multi_widget_init');
	

// Add actions and filters (only in widgets admin page) 

	function nmp_multi_widget_admin_init() {
	
		global $pagenow;
		
		$load_editor = false;
		
		if ($pagenow == "widgets.php") {
		
			$load_editor = true;
			
		}
		
		// Compatibility for WP Page Widget plugin
		
		if (is_plugin_active('wp-page-widget/wp-page-widgets.php') && ( $pagenow == "post-new.php" ||  $pagenow == "post.php" )) {
		
			$load_editor = true;
			
		}		
		
		if ($load_editor) {
		
			add_action( 'admin_head', 'nmp_multi_widget_load_tiny_mce');
			add_filter( 'tiny_mce_before_init', 'nmp_multi_widget_init_editor', 20);
			add_action( 'admin_print_scripts', 'nmp_multi_widget_scripts');
			add_action( 'admin_print_styles', 'nmp_multi_widget_styles');
			add_action( 'admin_print_footer_scripts', 'nmp_multi_widget_footer_scripts');
		
		}
		
		
	}
	
	add_action('admin_init', 'nmp_multi_widget_admin_init');

// Load TinyMCE

	function nmp_multi_widget_load_tiny_mce() {
	
	// Remove filters added from "After the deadline" plugin, to avoid conflicts
	
		remove_filter( 'mce_external_plugins', 'add_AtD_tinymce_plugin' );
		remove_filter( 'mce_buttons', 'register_AtD_button' );
		remove_filter( 'tiny_mce_before_init', 'AtD_change_mce_settings' );

		// Add support for thickbox media dialog
		
		add_thickbox();
		
		// New media modal dialog (WP 3.5+)
		
		if (function_exists('wp_enqueue_media')) {
		
			wp_enqueue_media();
			
		}
	
	}

// Customize TinyMCE

	function nmp_multi_widget_init_editor($initArray) {
	
	// Remove WP fullscreen mode and set the native tinyMCE fullscreen mode
	
		if (get_bloginfo('version') < "3.3") {
		
			$plugins = explode(',', $initArray['plugins']);
			
			if (isset($plugins['wpfullscreen'])) {
			
				unset($plugins['wpfullscreen']);
				
			}
			
			if (!isset($plugins['fullscreen'])) {
			
				$plugins[] = 'fullscreen';
				
			}
			
			$initArray['plugins'] = implode(',', $plugins);
		}
		
	// Remove the "More" toolbar button
	
		$initArray['theme_advanced_buttons1'] = str_replace(',wp_more', '', $initArray['theme_advanced_buttons1']);
		
	// Do not remove linebreaks
	
		$initArray['remove_linebreaks'] = false;
		
	// Convert newline characters to BR tags
	
		$initArray['convert_newlines_to_brs'] = false; 
		
	// Force P newlines
	
		$initArray['force_p_newlines'] = true; 
		
	// Force P newlines
		
		$initArray['force_br_newlines'] = false; 
		
	// Do not remove redundant BR tags
		
		$initArray['remove_redundant_brs'] = false;
		
	// Force p block
		
		$initArray['forced_root_block'] = 'p';
		
	// Apply source formatting
		
		$initArray['apply_source_formatting '] = true;
		
	// Return modified settings
		
		return $initArray;
		
	}


// Load JS

	function nmp_multi_widget_scripts() {
	
		global $multi_widget_version;
		
		wp_enqueue_script('media-upload');	
		
		add_thickbox();
		
		if (get_bloginfo('version') >= "3.3") {
		
			wp_enqueue_script('wplink');
			wp_enqueue_script('wpdialogs-popup');
			wp_enqueue_script('nmp_multi_widget', get_template_directory_uri() . '/functions/widgets/widget-multi/multi-widget-3.5.js', array('jquery'), $multi_widget_version);

		} else {
		
			wp_enqueue_script('nmp_multi_widget', get_template_directory_uri() . '/functions/widgets/widget-multi/multi-widget.js', array('jquery'), $multi_widget_version);

		}
		
		

	}

// Load CSS

	function nmp_multi_widget_styles() {

		global $multi_widget_version;
	
		if (get_bloginfo('version') < "3.3") {
		
			wp_enqueue_style('thickbox');
			
		} else {
		
			wp_enqueue_style('wp-jquery-ui-dialog');
			
		}
		
		wp_print_styles('editor-buttons');
		
		wp_enqueue_style('nmp_multi_widget', get_template_directory_uri() . '/functions/widgets/widget-multi/multi-widget.css', array(), $multi_widget_version);
	}


// Footer Script

	function nmp_multi_widget_footer_scripts() {

		// Setup for WP 3.1 and previous versions
		
		if (get_bloginfo('version') < "3.2") {
		
			if (function_exists('wp_tiny_mce')) {
			
				wp_tiny_mce(false, array());
				
			}
			
			if(function_exists('wp_tiny_mce_preload_dialogs')) {
			
				wp_tiny_mce_preload_dialogs();
				
			}
			
		}
		
		// Setup for WP 3.2.x
		
		else if (get_bloginfo('version') < "3.3") {
		
			if (function_exists('wp_tiny_mce')) {
			
				wp_tiny_mce(false, array());
				
			}
			
			if(function_exists('wp_preload_dialogs')) {
			
				wp_preload_dialogs( array( 'plugins' => 'wpdialogs,wplink,wpfullscreen' ) );
				
			}
			
		}
		
		// Setup for WP 3.3 - New Editor API
		
		else {
		
			wp_editor('', 'nmp_multi_widget');
			
		}
		
	}
	
	/* Hack needed to enable full media options when adding content form media library */
	/* (this is done excluding post_id parameter in Thickbox iframe url) */
	
	add_filter('_upload_iframe_src', 'nmp_multi_widget_upload_iframe_src');
	
	function nmp_multi_widget_upload_iframe_src ($upload_iframe_src) {
	
		global $pagenow;
		
		if ($pagenow == "widgets.php" || ($pagenow == "admin-ajax.php" && isset ($_POST['id_base']) && $_POST['id_base'] == "nmp_multi_widget") ) {
			
			$upload_iframe_src = str_replace('post_id=0', '', $upload_iframe_src);
			
		}
		
		return $upload_iframe_src;
		
	}	

?>