/*
	Treeonic 1.1: A general purpose expanding tree plugin
	By nodethirtythree design | http://nodethirtythree.com/ | @nodethirtythree
	Dual licensed under the MIT or GPLv2 license.
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	MIT LICENSE:
	Copyright (c) 2012 nodethirtythree design, http://nodethirtythree.com/
	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
	copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:
	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	GPLv2 LICENSE:
	Copyright (c) 2012 nodethirtythree design, http://nodethirtythree.com/
	This program is free software: you can redistribute it and/or modify it	under the terms of the GNU General Public License as
	published by the Free Software Foundation, either version 2 of the License, or (at your option) any later version. This program is
	distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
	or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of
	the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>. 
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
*/

(function(jQuery) {
	
	jQuery.fn.treeonic = function(options) {
		var settings = jQuery.extend({
			selectorParent:		jQuery(this)
		}, options);
		return jQuery.treeonic(settings);
	}
	
	jQuery.fn.treeonic_disableSelection = function() { return jQuery(this).css('user-select', 'none').css('-khtml-user-select', 'none').css('-moz-user-select', 'none').css('-o-user-select', 'none').css('-webkit-user-select', 'none'); }
	
	jQuery.treeonic = function(options) {
		// Settings
			var settings = jQuery.extend({
				selectorParent:			null,			// If a jQuery object, all selectors will be restricted to its scope. Otherwise, all selectors will be global.
				baseZIndex:				1,				// Base Z-Index
				initialSelector:		'.initial',		// Initial list item selector
				exclusive:				false,			// If true, expanding one list collapses all others.
				expandOnHover:			false,			// If true, hovering an item will activate it (after <hoverDelay>ms)
				hoverDelay:				250,			// If expandOnHover is true, this determines how long to wait
				collapseOnBlur:			false,			// If true, clicking outside the list will collapse all open lists
				collapseOnLeave:		true,			// If true, moving the cursor outside the list will immediately collapse all open lists
				levelClassPrefix:		'level',		// Level class prefix
				mode:					'instant',		// Display mode ("instant", "fade", "slide")
				speed:					'fast'			// Opening speed (only when mode is "fade" or "slide")
			}, options);
		// Variables
			var _top = settings.selectorParent, x = _top.find('> li > ul'), y = x, i, timeoutId, z = settings.baseZIndex;
		// Main
			jQuery.fn.treeonic_show = function(forceMode) {
				var t = jQuery(this), m = (forceMode ? forceMode : settings.mode);
				switch (m) {
					case 'slide':
						t.slideDown(settings.speed);
						break;
					case 'fade':
						t.fadeIn(settings.speed);
						break;
					default:
						t.show();
						break;
				}
				return t;
			}
			jQuery.fn.treeonic_toggle = function(forceMode) {
				var t = jQuery(this);
				if (t.is(':visible'))
					t.hide();
				else
					t.treeonic_show(forceMode);
				return t;
			}
			jQuery().ready(function() {
				for(i=1;y.length > 0;i++) {
					y
						.css('z-index', z++)
						.addClass(settings.levelClassPrefix + i)
						.hide();
					y = y.find('> li > ul');
				}
				_top.treeonic_disableSelection();
				_top
					.bind('treeonic_expandInitial', function() {
						var x = _top.find(settings.initialSelector), y = x.find('> ul');
						x.addClass('active').parents().show();
						y.show();
					});
				_top
					.bind('treeonic_collapseAll', function() {
						_top.find('ul').hide(); _top.find('ul:hidden').parent().removeClass('active');
						_top.trigger('treeonic_expandInitial');
					});
				var ll = _top.find('li');
				ll.click(function(e, expandOnly, forceMode) {
					e.preventDefault();
					e.stopPropagation();
					var p = jQuery(this), q = p.find('> ul');
					if (q.length > 0) {
						if (expandOnly) {
							if (settings.exclusive)
								_top.trigger('treeonic_collapseAll');
							q.treeonic_show(forceMode);
							q.parents.show();
							p.addClass('active');
						}
						else
						{
							if (settings.exclusive) {
								_top.trigger('treeonic_collapseAll');
								q.treeonic_show(forceMode);
								q.parents().show();
								p.addClass('active');
							}
							else {
								q.treeonic_toggle(forceMode);
								q.parents().show();
								p.toggleClass('active');
							}
						}
					}
					return false;
				});
				if (settings.expandOnHover)
					_top.find('li').hover(
						function(e) {
							var t = jQuery(this);
							timeoutID = window.setTimeout(function() {
								t.trigger('click', true);
							}, settings.hoverDelay);
						},
						function (e) {
							window.clearTimeout(timeoutID);
						}
					);
				if (settings.collapseOnBlur)
					jQuery(window).click(function() {
						_top.trigger('treeonic_collapseAll'); 
					});
				if (settings.collapseOnLeave)
					_top.mouseleave(function() {
						_top.trigger('treeonic_collapseAll'); 
					});
				_top.find('a').click(function(e) {
					e.stopPropagation();
				});
				window.setTimeout(function() {
					_top.trigger('treeonic_expandInitial');
				}, 50);
			});
	}
})(jQuery);